--[[ **Dialogue System** ]]
Import "DialogueData"
Import "EventScripts"

DialogueQueue = {}

function DialogueProcDisplay()

	if dialogueProcSequence or #DialogueQueue == 0 then
		return
	end

	dialogueProcSequence = true

	-- hide location UI if it's showing
	HideLocationText()

	ClearCaravanText()

	if introComplete then
		SetOvermapMusic( "dialogue" )
	end

	local showingDialogue = HasFlag({ FlagSet = "LOCAL_SCRIPTS", Name = "ShowingDialogue" })
	assert( not showingDialogue )

	SetFlagTrue({ Name = "ShowingDialogue" })

	local alreadyOpen = OpenEventScreen()
	if alreadyOpen then
		SetMenuOptions({ Name = "DialogueScreen", Item = "ConfirmButton",
			Properties = { UnderCursorOnAnimation = "Default", UnderCursorOffAnimation = "Default", PressAnimation = "DialogueContinueButtonPress", Enabled = true, IsVisible = true } })
		SetMenuOptions({ Name = "DialogueScreen", Item = "ConfirmButton", Properties = { Graphic = "DialogueContinueButtonIn" } })
	end

	DarkenBackground()
	HideOptions()

	-- bloom setting
	if not narrationBloom and not preDialogueBloom and not wagonNarrativeSequence then
		preDialogueBloom = true
		preDialogueBloomSetting = GetBloomSettingName({ })
		AdjustFullscreenBloom({ Name = "Blurry", Duration = 0.35, UseWorldTime = false })
	end

	local sceneCharacters = { }

	for _, dialogueData in pairs(DialogueQueue) do
		local sceneData = dialogueData.Scene
		if sceneData ~= nil then
			GetStoryTaggedForScene( sceneData )
			for _,charPositionData in ipairs(sceneData) do
				sceneCharacters[charPositionData.Position] = true
			end
		end
		local lines = dialogueData.Lines
		SetupScene( dialogueData, DialoguePositions )

		waitScreenTime(0.05)

		ShowSpeechBackground()
		local finalLine = false

		-- play dialogue lines
		local lineCount = #lines
		for k, lineData in ipairs( lines ) do
			lineData = GetConditionalLineData( lineData )
			if lineData ~= nil then
				local characterData = GetCharacter( lineData )
				local displaySide = CharacterPositions[characterData]
				local displaySideOverride = lineData.Position
				if displaySideOverride ~= nil then
					if string.find( displaySideOverride, "Dynamic" ) ~= nil then
						displaySideOverride = GetDynamicHeightPosition( sceneData, characterData, displaySideOverride )
					end
					lineData.StageDirection = "Enter"
					displaySide = displaySideOverride
				end

				if k == lineCount and IsDialogueEndEvent() then
					finalLine = true
				end

				DisplayDialogueLine( characterData, displaySide, lineData, finalLine )

				waitUntil("DialogueContinue")

				-- Check stage direction
				if lineData.StageDirection ~= nil and string.find( lineData.StageDirection, "Exit" ) ~= nil then
					SceneCharacterExit( displaySide, lineData )
				end

				if finalLine then
					SetMenuOptions({ Name = "DialogueScreen", Item = "ConfirmButton", Properties = { Enabled = false } })
				end

				waitScreenTime(0.06)

				TriggerChar = { }

				waitScreenTime(0.01)
			end
		end
	end

	waitScreenTime(0.08)

	HideNamePlate()
	HideSpeechBackground()

	if DialogueFollowUpLockerRoom == nil then

		-- get characters exited
		if sceneCharacters["LeftMidCharacter"] then
			SceneCharacterExit( "LeftMidCharacter" )
		end
		if sceneCharacters["RightMidCharacter"] then
			SceneCharacterExit( "RightMidCharacter" )
		end

		if sceneCharacters["LeftMidCharacter"] or sceneCharacters["RightMidCharacter"] then
			waitScreenTime(0.05)
		end

		if sceneCharacters["LeftCharacter"] then
			SceneCharacterExit( "LeftCharacter" )
		end
		if sceneCharacters["RightCharacter"] then
			SceneCharacterExit( "RightCharacter" )
		end

		SetMenuOptions({ Name = "DialogueScreen", Item = "DialogueBoxBack", Properties = { Graphic = "DialogueBackgroundOut" } })
		SetMenuOptions({ Name = EventScreenName, Item = "DialogueBoxDecoration", Properties = { Graphic = "DialogueLampOut" } })
		SetMenuOptions({ Name = EventScreenName, Item = "CharacterSplitter", Properties = { FadeTarget = 0.0 } })
	end

	waitScreenTime(0.08)

end

OnMenuButtonPressed{ RequiredFlag = "ShowingDialogue", function( triggerArgs )

	if triggerArgs.name == nil then
		return
	end

	if GetTopScreenName({ }) ~= "DialogueScreen" then
		return
	end

	if triggerArgs.name ~= "ConfirmButton" then
		return
	end

	if dialogueTyping then
		-- Hurry the text if it's not done yet
		HurryDialogueBox()
		StopSound({ Name = lastCueName, Duration = 0.65 })
		return
	end

	notifyExistingWaiters("DialogueContinue")

end }

function DialogueProcSequence()

	if IsEmpty( DialogueQueue ) then
		return
	end

	-- using map to know context for now
	local currentDialogueMap = GetMapName({ })

	if currentDialogueMap ~= "Campaign" and not hidingUI then
		hidingUI = true
		hideEntireUI()
	end

	-- loop through everyone with a status effect
	DialogueProcDisplay()
	DialogueProcSequenceCompleted()

	if hidingUI then
		hidingUI = false
		--ShowMatchUI() -- not necessary any more [AR 3/25]
	end

	if DialogueFollowUpLockerRoom ~= nil then
		local lockerRoomName = DialogueFollowUpLockerRoom
		DialogueFollowUpLockerRoom = nil
		NarrationSequence( lockerRoomName )
	end

	notifyExistingWaiters("DialogueSequenceComplete")

end

function DialogueProcSequenceCompleted()

	SetFlagFalse({ Name = "ShowingDialogue" })

	dialogueProcSequence = false
	DialogueQueue = { }

	-- with 'if not openedCaravan' here the bloom stays broken after every lobby conversation
	--if preDialogueBloom and not openedCaravan then
	if preDialogueBloom then
		preDialogueBloom = false
		AdjustFullscreenBloom({ Name = preDialogueBloomSetting, Duration = 0.25 })
	end

	if DialogueFollowUpLockerRoom == nil then
		CloseEventScreen()

		if GetTopScreenName({ }) == "LocationMenuScreen" and introComplete then
			EnableInput({ })
		end
		if not openedCaravan then
			SetOvermapMusic( "mapidle" )
		end
	else
		SetOvermapMusic( "narration" )
	end

end

function AddDialogueByName( dialogueName, blockRepeats )

	if dialogueName == nil then
		return
	end

	local dialogueData = DialogueData[dialogueName]

	if dialogueData == nil then
		return
	end

	if blockRepeats == nil then
		blockRepeats = true
	end

	if blockRepeats and DialogueByNameRecord[dialogueName] then
		return
	end

	table.insert( DialogueQueue, dialogueData )

	DialogueFollowUpLockerRoom = dialogueData.FollowUpLockerRoom

	DialogueByNameRecord[dialogueName] = true
	notifyExistingWaiters( "DialogueByNameRecordUpdated" )

end

function AddDialogue( character, speakerLine, quipType, soundName, displaySide, stageDirection, lineData )

	-- quipType options: "FiredUp", "Determined", "Demoralized", "Taunt", "Concede", "Excited", "Bark", "Surprised", "Sardonic", "Giddy", "Pleased", "Angry", "Sad", "Skeptical", "Curious", "Grateful", "Sorry", "Mocking", "Naive", "Tough"

	local leftBacking = nil
	local rightBacking = nil
	if lineData ~= nil then
		leftBacking = lineData.LeftBacking
		rightBacking = lineData.RightBacking
	end

	local dialogueData = {
		Lines = {
			{
				Character = character,
				HelpTextId = speakerLine,
				QuipType = quipType,
				SoundName = soundName,
				StageDirection = stageDirection,
				LeftBacking = leftBacking,
				RightBacking = rightBacking,
			}
		},
		Scene = {
			Character = character,
			Position = displaySide or "RightCharacter"
		}
	}
	table.insert( DialogueQueue, dialogueData )

	DialogueFollowUpLockerRoom = nil

end

-- **SCRIPTED DIALOGUE**

-- **OVERMAP DIALOGUE**

-- **MATCH DIALOGUE**
function PlayMatchDialogue( args )

	-- args: prematch; postmatch
	if args == nil then
		return
	end

	-- **context list** - in chronological order
	-- prematch
	-- tutorialstart
	-- tutorialend
	-- draftstart
	-- draftscreenopen
	-- draftpick
	-- draftend
	-- matchstart
	-- midmatch
	-- postpowerup
	-- postmatch
	-- note: Match Conclusion Events in PostMatchScripts

	-- **for local mp**
	if IsMultiplayerMatch() then
		if args == "postmatch" then
			if TeamA.PyreHealth > TeamB.PyreHealth then
				local victoryEvent = GetRandomRemainingEvent( TeamA.LocalMPTeamAVictoryEvents )
				LocalMPWinningTeam = TeamA
				NarrationSequence( victoryEvent )
			else
				local victoryEvent = GetRandomRemainingEvent( TeamB.LocalMPTeamBVictoryEvents )
				LocalMPWinningTeam = TeamB
				NarrationSequence( victoryEvent )
			end
		end
		return
	end

	-- first-time loss
	if args == "postmatch" and PlayerTeam.TeamLastMatchResult == "Loss" then
		if not uncommonMatch and not practiceMatch and PlayerTeam.AscensionMatchesPlayed ~= 7 then
			NarrationSequenceOnce( "Match_FirstTimeLoss" )
		end
	end

	-- **for intro match**
	if introMatch then
		if not introComplete then
			if args == "prematch" then
				NarrationSequence( "Intro_IntroMatchStart")
			elseif args == "postmatch" then
				AddDialogueByName( "IntroMatchEndDialogue" )
			end
		end
	end

	if introMatch then
		return
	end

	-- **for practice matches**
	if practiceMatch then

		-- **for freeplay matches**
		if ChallengeData == nil then
			if freePlayMatchesCompleted == 0 or freePlayMatchesCompleted == nil then
				if args == "prematch" then
					if CharFriend["RosterStatus"] ~= "Ascended" and CharBadass["RosterStatus"] ~= "Ascended" and CharJoker["RosterStatus"] ~= "Ascended" then
						NarrationSequenceOnce( "FreePlayMatch_01_Beginning" )
					else
						if not NarrationRecord["FreePlayMatch_01_Beginning"] then
							NarrationSequenceOnce( "FreePlayMatch_01_Beginning_Alt" )
						end
					end
				elseif args == "draftend" then
					NarrationSequenceOnce( "FreePlayMatch_01_DraftEnd" )
				end
			elseif args == "postmatch" then
				if not NarrationRecord["RevealOpenStructure"] then
					-- NarrationSequenceOnce( "FreePlayMatch_01_Conclusion" )
				end
			end
		-- **for challenge matches**
		else
			if args == "prematch" then
				if not IsEmpty( ChallengeCharacter["ChallengeIntroEvents"] ) then
					eventName = RemoveFirstIndexValue( ChallengeCharacter["ChallengeIntroEvents"] )
					NarrationSequence( eventName )
				end
			elseif args == "postmatch" then
				-- if you won the challenge
				if TeamA.PyreHealth > TeamB.PyreHealth then
					local characterData = PlayerData.Teams[1].TeamBench[ChallengeCharacter.CharacterIndex]
					if not IsEmpty( characterData["ChallengeSuccessEvents"] ) then
						eventName = RemoveFirstIndexValue( characterData["ChallengeSuccessEvents"] )
						NarrationSequence( eventName )
					end
				-- if you failed the challenge
				else
					if not IsEmpty( ChallengeCharacter["ChallengeFailureEvents"] ) then
						eventName = RemoveFirstIndexValue( ChallengeCharacter["ChallengeFailureEvents"] )
						NarrationSequence( eventName )
					else
						if not retryDialogueDisplayed then
							retryDialogueDisplayed = true
							PersistVariable({ Name = "retryDialogueDisplayed" })
							-- this version has Sandra dialogue
							NarrationSequence( "ChallengeMatch_Failure_Retry" )
						else
							-- this version is quicker
							NarrationSequence( "ChallengeMatch_Failure_Retry_Quick" )
						end
					end
				end
			end

		end

		return
	end

	-- **match dialogue**
	if not uncommonMatch then

		-- Eight Weeks
		if matchesPlayed < 8 then

			-- Week 1: Ridge match
			if GetMapName({ }) == "MatchSiteB" then
				-- beginning
				if args == "prematch" then
					NarrationSequence( "Intro_Week1_Match_Beginning" )
				-- middle
				elseif args == "midmatch" then
				--
				elseif args == "postpowerup" then
					if TeamA.NumScores ~= nil and TeamA.NumScores == 2 then
						NarrationSequenceOnce( "Week1_Match_PostPowerUp_Winning" )
					else
						NarrationSequenceOnce( "Week1_Match_PostPowerUp_Losing" )
					end
				-- end
				elseif args == "postmatch" then
					if lastMatchResult == "Win" then
						NarrationSequence( "Week1_Match_YouWon" )
					else
						NarrationSequence( "Week1_Match_YouLost" )
					end
				end

			-- Week 2: Spring match
			elseif GetMapName({ }) == "MatchSiteC" then
				-- beginning
				if args == "prematch" then
					NarrationSequence( "Week2_Match_Beginning" )
				-- draft start
				elseif args == "draftstart" then
					NarrationSequence( "Week2_Match_DraftStart" )
				-- draft screen open
				-- elseif args == "draftscreenopen" then
				--	NarrationSequence( "Week2_Match_TalismanIntro" )
				-- draft pick
				elseif args == "draftpick" then
					for index, character in ipairs( PlayerTeam["TeamBench"] ) do
						if character["ActiveStatus"] == "Assigned" then
							if character == CharFriend then
								AddDialogueByName( "S1W2_DraftPickDialogue_Friend" )
							elseif character == CharBadass then
								AddDialogueByName( "S1W2_DraftPickDialogue_Badass" )
							elseif character == CharJoker then
								AddDialogueByName( "S1W2_DraftPickDialogue_Joker" )
							elseif character == CharPsycho then
								AddDialogueByName( "S1W2_DraftPickDialogue_Psycho" )
							end
						end
					end
				-- draft end
				elseif args == "draftend" then
					for index, character in ipairs( PlayerTeam["TeamBench"] ) do
						if character["ActiveStatus"] ~= "Assigned" then
							if character == CharFriend then
								AddDialogueByName( "S1W2_DraftBenchDialogue_Friend" )
							elseif character == CharBadass then
								AddDialogueByName( "S1W2_DraftBenchDialogue_Badass" )
							elseif character == CharJoker then
								AddDialogueByName( "S1W2_DraftBenchDialogue_Joker" )
							elseif character == CharPsycho then
								AddDialogueByName( "S1W2_DraftBenchDialogue_Psycho" )
							end
						end
					end
				-- middle
				elseif args == "midmatch" then
					--[[ removed for now, GK 8/30/16
					if TeamB.PyreHealth <= 60 and TeamB.PyreHealth > 30 and TeamA.PyreHealth > 60 then
						AddDialogueByName( "S1W2_MidMatchDialogue_Winning" )
					elseif TeamA.PyreHealth <= 60 and TeamA.PyreHealth > 30 and TeamB.PyreHealth > 60 then
						AddDialogueByName( "S1W2_MidMatchDialogue_Losing" )
					end
					]]--
				-- end
				elseif args == "postmatch" then
					if lastMatchResult == "Win" then
						NarrationSequence( "Week2_Match_YouWon" )
					else
						NarrationSequence( "Week2_Match_YouLost" )
					end
				end

			-- Week 3: Canyon / Cairn match
			elseif GetMapName({ }) == "MatchSiteD" then
				-- beginning
				if args == "prematch" then
					NarrationSequence( "Week3_Match_Beginning" )
				-- ti'zo tutorial
				elseif args == "tutorialstart" then
					NarrationSequence( "Week3_Match_TizoTestDrive" )
				elseif args == "draftstart" then
					NarrationSequence( "Week3_Match_DraftStart" )
				-- draft end
				elseif args == "draftend" then
					NarrationSequence( "Week3_Match_DraftEnd" )
				-- end
				elseif args == "postmatch" then
					if lastMatchResult == "Win" then
						NarrationSequence( "Week3_Match_YouWon" )
					else
						NarrationSequence( "Week3_Match_YouLost" )
					end
				end

			-- Week 4: Lair / Pit match
			elseif GetMapName({ }) == "MatchSiteE" then
				-- beginning
				if args == "prematch" then
					NarrationSequence( "Week4_Match_Beginning" )
				-- draft end
				elseif args == "draftend" then
					NarrationSequence( "Week4_Match_DraftEnd" )
				-- end
				elseif args == "postmatch" then
					if lastMatchResult == "Win" then
						NarrationSequence( "Week4_Match_YouWon" )
					else
						NarrationSequence( "Week4_Match_YouLost" )
					end
				end

			-- Week 5: Ship / Hulk match
			elseif GetMapName({ }) == "MatchSiteF" then
				-- beginning
				if args == "prematch" then
					NarrationSequence( "Week5_Match_Beginning" )
				-- draft start
				elseif args == "draftstart" then
					NarrationSequence( "Week5_Match_DraftStart" )
				-- draft end
				elseif args == "draftend" then
					NarrationSequence( "Week5_Match_DraftEnd" )
				-- end
				elseif args == "postmatch" and GetMapName({ }) == "MatchSiteF" then
					if lastMatchResult == "Win" then
						NarrationSequence( "Week5_Match_YouWon" )
					else
						NarrationSequence( "Week5_Match_YouLost" )
					end
				end

			-- Week 6: Lava / Nest match
			elseif GetMapName({ }) == "MatchSiteG" then
				-- beginning
				if args == "prematch" then
					NarrationSequence( "Week6_Match_Beginning" )
				-- draft start
				elseif args == "draftstart" then
					NarrationSequence( "Week6_Match_DraftStart" )
				-- draft pick
				elseif args == "draftpick" then
					NarrationSequence( "Week6_Match_DraftPick" )
					if not NarrationRecord["Week6_Match_DraftPick_JodiFollowUp"] then
						NarrationSequence( "Week6_Match_DraftPick_JodiFollowUp" )
					end
				-- draft end
				elseif args == "draftend" then
					NarrationSequence( "Week6_Match_DraftEnd" )
				-- end
				elseif args == "postmatch" then
					if lastMatchResult == "Win" then
						NarrationSequence( "Week6_Match_YouWon" )
					else
						NarrationSequence( "Week6_Match_YouLost" )
					end
				end

			-- Week 7: Woods / Glade match
			elseif GetMapName({ }) == "MatchSiteH" then
				-- beginning
				if args == "prematch" then
					NarrationSequence( "Week7_Match_Beginning" )
				-- draft end
				elseif args == "draftend" then
					NarrationSequence( "Week7_Match_DraftEnd" )
				-- end
				elseif args == "postmatch" and GetMapName({ }) == "MatchSiteH" then
					if lastMatchResult == "Win" then
						NarrationSequence( "Week7_Match_YouWon" )
					else
						NarrationSequence( "Week7_Match_YouLost" )
					end
				end

			-- Week 8: Ascension / Fall match
			elseif GetMapName({ }) == "MatchSiteI" then
				-- beginning
				if args == "prematch" then
					GetEntireTeam()
					NarrationSequence( "Week8_Match_Beginning" )
				-- draft start
				elseif args == "draftstart" then
					if GetNumCharsAtLevelCap( PlayerTeam ) < 3 then
						NarrationSequence( "Week8_Match_DraftStart_Alt" )
					else
						NarrationSequence( "Week8_Match_DraftStart" )
					end
				-- draft pick
				elseif args == "draftpick" then
					if not choseAscensionCharacter then
						NarrationSequence( "Week8_Match_DraftPick_Ascension" )
						choseAscensionCharacter = true
					else
						NarrationSequence( "Week8_Match_DraftPick" )
					end
				-- draft end
				elseif args == "draftend" then
					NarrationSequence( "Week8_Match_DraftEnd" )
				end
			end

		-- Week 8+: Open Structure
		else

			local eventName = nil
			-- beginning
			if args == "prematch" then
				-- pre-match events based on team you are facing: season matches
				if not ascensionMatch then
					-- team-specific rivalry stories (if any)
					if IsRivalryPresent() and not IsEmpty( NextTeam["RivalryMatchStartEvents"] ) then
						-- if the rivalry character is active (or available in Volfred's case)
						eventName = RemoveFirstIndexValue( NextTeam["RivalryMatchStartEvents"] )
						NarrationSequence( eventName )
						return
					end
					-- last season match
					if IsLastSeasonMatch() then
						eventName = "OpenStructure_LastSeasonMatch_Beginning_01"
						NarrationSequence( eventName )
						return
					end
					-- for team that got bumped by True Nightwings
					if NextTeam.GotGanked and ( NextTeam["AscensionMatchesPlayed"] == NextTeam["AscensionMatchesPlayedWhenGanked"] )then
						eventName = "OpenStructure_GankedMatch_Beginning_01"
						NarrationSequence( eventName )
						NextTeam.GotGanked = nil
						return
					end
					-- if the rivalry character in unavailable / ascended
					if IsRivalryEnded() and not IsEmpty( NextTeam["RivalryEndedMatchStartEvents"] ) then
						eventName = RemoveFirstIndexValue( NextTeam["RivalryEndedMatchStartEvents"] )
						NarrationSequence( eventName )
						return
					end
					-- grudge match: if you've confronted the team in an ascension match before
					if NextTeam["AscensionMatchesPlayed"] ~= nil and NextTeam["AscensionMatchesPlayed"] > 0 and not NextTeam["GrudgeMatchOccurred"] then
						NextTeam["GrudgeMatchOccurred"] = true
						grudgeMatchEvent = true
						-- search for team-specific events
						if not IsEmpty( NextTeam["GrudgeMatchStartEvents"] ) then
							eventName = GetNextRemainingEvent( NextTeam["GrudgeMatchStartEvents"] )
							NarrationSequence( eventName )
							return
						-- choose a general event if no team-specific events found
						elseif not IsEmpty( PlayerTeam["GrudgeMatchStartEvents"] ) then
							eventName = GetNextRemainingEvent( PlayerTeam["GrudgeMatchStartEvents"] )
							NarrationSequence( eventName )
							return
						end
					end
					-- team-specific story events (if any)
					if NextTeam["MatchStartEvents"] ~= nil then
						eventName = GetNextRemainingEvent( NextTeam["MatchStartEvents"] )
						NarrationSequence( eventName )
						return
					-- general match start events
					elseif PlayerTeam["MatchStartEvents"] ~= nil then
						eventName = GetNextRemainingEvent( PlayerTeam["MatchStartEvents"] )
						NarrationSequence( eventName )
						return
					end
				-- pre-match events: ascension matches
				else
					if args == "prematch" then
						-- Final Liberation / True Nightwings exception
						if PlayerTeam.AscensionMatchesPlayed == 6 then
							NarrationSequence( "Match_Ascension7_Beginning" )
							return
						elseif not IsEmpty( NextTeam["AscensionMatchStartEvents"] ) then
							eventName = RemoveFirstIndexValue( NextTeam["AscensionMatchStartEvents"] )
							NarrationSequence( eventName )
							return
						elseif not IsEmpty( PlayerTeam["AscensionMatchStartEvents"] ) then
							eventName = GetNextRemainingEvent( PlayerTeam["AscensionMatchStartEvents"] )
							NarrationSequence( eventName )
							return
						end
					end
				end
			-- open structure: tutorials
			elseif args == "tutorialstart" then
				-- Bertrude Tutorial
				if PlayerTeam.AscensionMatchesPlayed == 2 and MatchesPlayedThisSeason == 0 then
					NarrationSequence( "OpenStructure_Match_BertrudeTestDrive_Start" )
				-- Volfred Tutorial
				elseif PlayerTeam.AscensionMatchesPlayed == 3 and MatchesPlayedThisSeason == 0 then
					NarrationSequence( "OpenStructure_Match_VolfredTestDrive_Start" )
				end
			elseif args == "tutorialend" then
			-- open structure: draft start
				-- Bertrude Tutorial
				if PlayerTeam.AscensionMatchesPlayed == 2 and MatchesPlayedThisSeason == 0 then
					NarrationSequence( "OpenStructure_Match_BertrudeTestDrive_End" )
				-- Volfred Tutorial
				elseif PlayerTeam.AscensionMatchesPlayed == 3 and MatchesPlayedThisSeason == 0 then
					NarrationSequence( "OpenStructure_Match_VolfredTestDrive_End" )
				end
			elseif args == "draftstart" then
				-- draft start: rivalry season matches
				if not IsEmpty( NextTeam["RivalryMatchDraftStartEvents"] ) and IsRivalryPresent() and not ascensionMatch then
					-- Withdrawn special case
					if ConstellationActive( "RivalTeamAI" ) and not IsEmpty( NextTeam["RivalryMatchDraftStartAltEvents"] ) then
						eventName = GetRandomValue( NextTeam["RivalryMatchDraftStartAltEvents"] )
						NarrationSequenceOnce( eventName )
						return
					else
						-- skip this if Difficulty Modifiers haven't been revealed yet
						if NarrationRecord["DifficultyModifiers_01"] then
							eventName = RemoveFirstIndexValue( NextTeam["RivalryMatchDraftStartEvents"] )
							NarrationSequence( eventName )
							return
						end
					end
				-- draft start: standard matches
				elseif not IsAscensionWeek() and not IsEmpty( NextTeam["MatchDraftStartEvents"] ) then
					eventName = RemoveFirstIndexValue( NextTeam["MatchDraftStartEvents"] )
					NarrationSequence( eventName )
					return
				-- draft start: ascension matches
				elseif ascensionMatch then
					-- Ascension 2
					if PlayerTeam.AscensionMatchesPlayed == 1 then
						eventName = "OpenStructure_Match_Ascension2_DraftStart"
					-- Ascension 3
					elseif PlayerTeam.AscensionMatchesPlayed == 2 then
						eventName = "OpenStructure_Match_Ascension3_DraftStart"
					-- Ascension 4
					elseif PlayerTeam.AscensionMatchesPlayed == 3 then
						eventName = "OpenStructure_Match_Ascension4_DraftStart"
					-- Ascension 5
					elseif PlayerTeam.AscensionMatchesPlayed == 4 then
						eventName = "OpenStructure_Match_Ascension5_DraftStart"
					-- Ascension 6
					elseif PlayerTeam.AscensionMatchesPlayed == 5 then
						eventName = "OpenStructure_Match_Ascension6_DraftStart"
					-- Final Liberation / Ascension 7
					elseif PlayerTeam.AscensionMatchesPlayed == 6 then
						eventName = "Match_Ascension7_DraftStart"
					end
					NarrationSequence( eventName )
					return
				end
			-- draft screen appears (after draftstart, before draftpick)
			elseif args == "draftscreenopen" then
				-- ascension match variants
				if ascensionMatch then
					if PlayerTeam.AscensionMatchesPlayed == 6 then
						if CharDoc.RosterStatus == "Ascended" then
							NarrationSequenceOnce( "Match_Ascension7_DraftScreenOpen_Alt" )
						else
							NarrationSequenceOnce( "Match_Ascension7_DraftScreenOpen" )
						end
						return
					end
				-- other instances
				else
					if not practiceMatch then
						if matchesPlayed == 8 then
							AddInputBlock({ Name = "EnemyUpgradePresentation" })
							SetMenuOptions({ Name = "RosterScreen", Item = "ConfirmButton", Properties = { IsVisible = false } })
							SetMenuOptions({ Name = "RosterScreen", Item = "LeftArrow", Properties = { IsVisible = false } })
							SetMenuOptions({ Name = "RosterScreen", Item = "RightArrow", Properties = { IsVisible = false } })
							HighlightRosterAdversaryButton()
							wait(0.45)
							NarrationSequence( "OpenStructure_Match_DraftScreenOpen_RivalsStronger" )
							DisplayTutorialBox({ HelpTextId = "Info_AdversaryStronger", X = 1370, Y = 930, Animation = "TutorialBoxSmallIn", Arrow = "Right" })
							SetMenuOptions({ Name = "RosterScreen", Item = "ConfirmButton", Properties = { IsVisible = true } })
							SetMenuOptions({ Name = "RosterScreen", Item = "LeftArrow", Properties = { IsVisible = true } })
							SetMenuOptions({ Name = "RosterScreen", Item = "RightArrow", Properties = { IsVisible = true } })
							RemoveInputBlock({ Name = "EnemyUpgradePresentation" })
							return
						elseif IsRivalryPresent() and not IsEmpty( NextTeam["RivalryMatchDraftScreenOpenEvents"] ) and not ascensionMatch then
							AddInputBlock({ Name = "DraftScreenOpenInterrupt" })
							SetMenuOptions({ Name = "RosterScreen", Item = "ConfirmButton", Properties = { IsVisible = false } })
							SetMenuOptions({ Name = "RosterScreen", Item = "LeftArrow", Properties = { IsVisible = false } })
							SetMenuOptions({ Name = "RosterScreen", Item = "RightArrow", Properties = { IsVisible = false } })
							wait(0.45)
							eventName = RemoveFirstIndexValue( NextTeam["RivalryMatchDraftScreenOpenEvents"] )
							NarrationSequence( eventName )
							SetMenuOptions({ Name = "RosterScreen", Item = "ConfirmButton", Properties = { IsVisible = true } })
							SetMenuOptions({ Name = "RosterScreen", Item = "LeftArrow", Properties = { IsVisible = true } })
							SetMenuOptions({ Name = "RosterScreen", Item = "RightArrow", Properties = { IsVisible = true } })
							RemoveInputBlock({ Name = "DraftScreenOpenInterrupt" })
							return
						end
					end
				end
			-- ascension match draft picking
			elseif args == "draftpick" then
				if ascensionMatch then
					-- Ascension: First Draft Picks
					if not choseAscensionCharacter then
						-- Final Liberation: first pick
						if PlayerTeam.AscensionMatchesPlayed == 6 then
							NarrationSequence( "Match_Ascension7_DraftPick_First" )
							choseAscensionCharacter = true
							return
						-- other Ascension Matches
						else
							-- for characters who are getting their first shot at ascension
							if not SelectedCharacter["PreviouslyFailedToAscend"] then
								if not IsEmpty( PlayerTeam["AscensionFirstDraftPickEvents"] ) then
									eventName = GetNextRemainingEvent( PlayerTeam["AscensionFirstDraftPickEvents"] )
									NarrationSequence( eventName )
									choseAscensionCharacter = true
									return
								end
							-- for characters who've failed to ascend before
							else
								if SelectedCharacter["TimesFailedToAscend"] ~= nil and SelectedCharacter["TimesFailedToAscend"] <= 2 then
									if not IsEmpty( PlayerTeam["AscensionFirstDraftPick_SecondChanceEvents"] ) then
										eventName = GetNextRemainingEvent( PlayerTeam["AscensionFirstDraftPick_SecondChanceEvents"] )
									end
								else
									if not IsEmpty( PlayerTeam["AscensionFirstDraftPick_RepeatedFailures"] ) then
										eventName = GetNextRemainingEvent( PlayerTeam["AscensionFirstDraftPick_RepeatedFailures"] )
									end
								end
								NarrationSequence( eventName )
								choseAscensionCharacter = true
								return
							end
						end
					-- Ascension: Support Picks
					else
						-- Final Liberation / Ascension 7
						if PlayerTeam.AscensionMatchesPlayed == 6 then
							NarrationSequence( "Match_Ascension7_DraftPick_Next" )
							return
						-- Other Ascension Matches
						else
							-- for supporting characters: if the character up for ascension is getting their first shot
							if not LastAscendedCharacter["PreviouslyFailedToAscend"] then
								if not IsEmpty( SelectedCharacter["AscensionNextDraftPickEvents"] ) then
									eventName = GetNextRemainingEvent( SelectedCharacter["AscensionNextDraftPickEvents"] )
									NarrationSequence( eventName )
									return
								elseif not IsEmpty( PlayerTeam["AscensionNextDraftPickEvents"] ) then
									eventName = GetNextRemainingEvent( PlayerTeam["AscensionNextDraftPickEvents"] )
									NarrationSequence( eventName )
									return
								end
							-- for supporting characters: if the character up for ascension failed to ascend before
							else
								if not IsEmpty( SelectedCharacter["AscensionNextDraftPickRetryEvents"] ) then
									eventName = GetNextRemainingEvent( SelectedCharacter["AscensionNextDraftPickRetryEvents"] )
									NarrationSequence( eventName )
									return
								elseif not IsEmpty( PlayerTeam["AscensionNextDraftPick_SecondChanceEvents"] ) then
									eventName = GetNextRemainingEvent( PlayerTeam["AscensionNextDraftPick_SecondChanceEvents"] )
									NarrationSequence( eventName )
									return
								end
							end
						end
					end
				end
			-- draft end
			elseif args == "draftend" then
				if not ascensionMatch and not grudgeMatchEvent then
					-- search for team-specific events
					if IsLastSeasonMatch() then
						eventName = "OpenStructure_LastSeasonMatch_DraftEnd_01"
						NarrationSequence( eventName )
						return
					elseif IsForfeitMatch() then
						eventName = "OpenStructure_ForfeitMatch_DraftEnd_01"
						NarrationSequence( eventName )
						return
					elseif not IsEmpty( NextTeam["RivalryMatchDraftEndEvents"] ) and IsRivalryPresent() then
						eventName = RemoveFirstIndexValue( NextTeam["RivalryMatchDraftEndEvents"] )
						NarrationSequence( eventName )
						return
					elseif not IsEmpty( NextTeam["MatchDraftEndEvents"] ) then
						eventName = RemoveFirstIndexValue( NextTeam["MatchDraftEndEvents"] )
						NarrationSequence( eventName )
						return
					-- choose a general event if no team-specific events found
					elseif not IsEmpty( PlayerTeam["MatchDraftEndEvents"] ) then
						eventName = GetNextRemainingEvent( PlayerTeam["MatchDraftEndEvents"] )
						NarrationSequence( eventName )
						return
					end
				-- ascension match: rival captain prematch quips
				elseif ascensionMatch then
					-- Final Liberation / Ascension 7
					if PlayerTeam.AscensionMatchesPlayed == 6 then
						NarrationSequence( "Match_Ascension7_DraftEnd" )
						return
					elseif NextTeam["AscensionMatchesPlayed"] ~= nil then
						-- first time you face a team in an ascension match
						if NextTeam["AscensionMatchesPlayed"] == 0 then
							if not IsEmpty( PlayerTeam["AscensionMatchDraftEndEvents"] ) then
								eventName = GetNextRemainingEvent( PlayerTeam["AscensionMatchDraftEndEvents"] )
								NarrationSequence( eventName )
								return
							end
						-- for an ascension re-match
						else
							if not IsEmpty( PlayerTeam["AscensionReMatchDraftEndEvents"] ) then
								eventName = GetNextRemainingEvent( PlayerTeam["AscensionReMatchDraftEndEvents"] )
								NarrationSequence( eventName )
								return
							end
						end
					end
				end
			-- match start
			elseif args == "matchstart" then
				if PlayerTeam.AscensionMatchesPlayed == 2 and MatchesPlayedThisSeason == 1 then
					--thread(ShowInputDisabledPresentation)
					--AddInputBlock({ Name = "ObjectiveSystemIntro" })
					NarrationSequence( "OpenStructure_Match_ObjectiveSystemIntro")
					objectiveSystemUnlocked = true
					PersistVariable({ Name = "objectiveSystemUnlocked" })
					wait(0.5)
					rewardOjectivePicked = true -- for first time use this needs to be set so the objective doesn't repeat
					DisplayObjective( "DoubleKill" )
					wait(0.5)
					--RemoveInputBlock({ Name = "ObjectiveSystemIntro" })
					--thread(HideInputDisabledPresentation)
				end
			-- middle
			elseif args == "midmatch" then
			-- post-match
			elseif args == "postmatch" and not ascensionMatch then
				if lastMatchResult == "Win" then
					PostNormalMatchVictoryDialogue()
				else
					PostNormalMatchDefeatDialogue()
				end
			end
		end
		DialogueProcSequence()

		-- **for debug**
		if args == "debug" then
			AddDialogueByName( "DebugMatchDialogue" )
		end
		DialogueProcSequence()
		if DialogueByNameRecord ~= nil then
			DialogueByNameRecord["DebugMatchDialogue"] = nil
		end

	end

end

function PostNormalMatchVictoryDialogue()

	-- if you win non-ascension matches

	-- forfeit victory (no counterpart for losing)
	if forfeitMatch then
		NarrationSequence( "OpenStructure_ForfeitMatch_YouWon_01" )
		return
	end

	-- last season match (they lost, you won)
	if lastSeasonMatch then
		NarrationSequence( "OpenStructure_LastSeasonMatch_YouWon_01" )
		return
	end

	-- rival team specific first (they lost, you won)
	if not IsEmpty( LastTeam["RivalryMatchVictoryEvents"] ) and IsRivalryPresent( LastTeam ) then
		-- Withdrawn special case
		if ConstellationActive( "RivalTeamAI" ) and not IsEmpty( LastTeam["RivalryMatchVictoryAltEvents"] ) then
			RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryEvents"] )
			RemoveFirstIndexValue( LastTeam["RivalryMatchDefeatEvents"] )
			RemoveFirstIndexValue( LastTeam["RivalryMatchDefeatAltEvents"] )
			eventName = RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryAltEvents"] )
			NarrationSequence( eventName )
			return
		else
			RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryAltEvents"] )
			RemoveFirstIndexValue( LastTeam["RivalryMatchDefeatEvents"] )
			RemoveFirstIndexValue( LastTeam["RivalryMatchDefeatAltEvents"] )
			eventName = RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryEvents"] )
			NarrationSequence( eventName )
			return
		end
	end

	if not IsEmpty( LastTeam["MatchDefeatEvents"] ) then
		RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryEvents"] )
		RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryAltEvents"] )
		RemoveFirstIndexValue( LastTeam["RivalryMatchDefeatAltEvents"] )
		eventName = RemoveFirstIndexValue( LastTeam["MatchDefeatEvents"] )
		NarrationSequence( eventName )
		return
	end

	-- standard playerteam variant (you won, they lost)
	if not IsEmpty( PlayerTeam["MatchVictoryEvents"] ) then
		eventName = GetNextRemainingEvent( PlayerTeam["MatchVictoryEvents"] )
		NarrationSequence( eventName )
		return
	end

end

function PostNormalMatchDefeatDialogue()

	-- if you lose non-ascension matches

	-- last season match (they won, you lost)
	if lastSeasonMatch then
		NarrationSequence( "OpenStructure_LastSeasonMatch_YouLost_01" )
		return
	end

	-- rival team specific first (they won, you lost)
	if not IsEmpty( LastTeam["RivalryMatchDefeatEvents"] ) and IsRivalryPresent( LastTeam ) then
		-- Withdrawn special case
		if ConstellationActive( "RivalTeamAI" ) and not IsEmpty( LastTeam["RivalryMatchDefeatAltEvents"] ) then
			RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryEvents"] )
			RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryAltEvents"] )
			RemoveFirstIndexValue( LastTeam["MatchDefeatEvents"] )
			eventName = RemoveFirstIndexValue( LastTeam["RivalryMatchDefeatAltEvents"] )
			NarrationSequence( eventName )
			return
		else
			RemoveFirstIndexValue( LastTeam["RivalryMatchVictoryEvents"] )
			eventName = RemoveFirstIndexValue( LastTeam["RivalryMatchDefeatEvents"] )
			NarrationSequence( eventName )
			return
		end
	end

	if not IsEmpty( LastTeam["MatchVictoryEvents"] ) then
		eventName = RemoveFirstIndexValue( LastTeam["MatchVictoryEvents"] )
		NarrationSequence( eventName )
		return
	end

	-- standard playerteam variant (you lost, they won)
	if not IsEmpty( PlayerTeam["MatchDefeatEvents"] ) then
		eventName = GetNextRemainingEvent( PlayerTeam["MatchDefeatEvents"] )
		NarrationSequence( eventName )
		return
	end

end

-- DEBUG KEYS
OnControlPressed {"Throw", RequiredFlag = "EXPERIMENTAL",
	function(triggerArgs)
		if GetTopScreenName({ }) == "DialogueScreen" and cheating then

			DialogueQueue = { }

			SetMenuOptions({ Name = "DialogueScreen", Item = "DialogueBoxHeader", Properties = { FadeTarget = 0.0, FadeSpeed = 2 } })
			SetMenuOptions({ Name = "DialogueScreen", Item = "DialogueBox", Properties = { FadeTarget = 0.0, FadeSpeed = 2 } })

			-- go through queue and get characters exited
			for k, dialogueData in pairs(DialogueQueue) do

				dialogueProcSequence = true

				local characterData = dialogueData["Speaker"]
				if dialogueData["DisplaySide"] == "LeftMidCharacter" then
					SetMenuOptions({ Name = "DialogueScreen", Item = "LeftMidCharacter", Properties = { MoveTargetShiftX = -slideInDistanceBack } })
				elseif dialogueData["DisplaySide"] == "RightMidCharacter" then
					SetMenuOptions({ Name = "DialogueScreen", Item = "RightMidCharacter", Properties = { MoveTargetShiftX = slideInDistanceBack} })
				end
			end
			waitScreenTime(0.05)
			for k, dialogueData in pairs(DialogueQueue) do
				local characterData = dialogueData["Speaker"]
				if dialogueData["DisplaySide"] == "LeftCharacter" then
					SetMenuOptions({ Name = "DialogueScreen", Item = "LeftCharacter", Properties = { MoveTargetShiftX = -slideInDistance } })
				elseif dialogueData["DisplaySide"] == "RightCharacter" then
					SetMenuOptions({ Name = "DialogueScreen", Item = "RightCharacter", Properties = { MoveTargetShiftX = slideInDistance } })
				end

			end

			SetMenuOptions({ Name = "DialogueScreen", Item = "DialogueBoxBack", Properties = { FadeTarget = 0.0 } })
			waitScreenTime(0.08)
			DialogueProcSequenceCompleted()
			if DialogueFollowUpLockerRoom ~= nil then
				local lockerRoomName = DialogueFollowUpLockerRoom
				DialogueFollowUpLockerRoom = nil
				NarrationSequence( lockerRoomName )
			end

			notifyExistingWaiters("DialogueSequenceComplete")
		end
	end
}

function IsDialogueEndEvent()

	if DialogueFollowUpLockerRoom == nil or NarrationData[DialogueFollowUpLockerRoom] == nil or IsEmpty( NarrationData[DialogueFollowUpLockerRoom].Lines ) then
		return true
	end

	return false

end